/**************************************************************************************
Author:			David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:		encrypt_scripts.ms
Date:			12/08/11
Description:	This script will create encrypted MAXScript files.
***************************************************************************************/

try (destroyDialog encrypt_script_rollout) catch()


struct EncryptScripts
(
    maxscript_files = #(),
    display_names = #(),
    
    /**************************************************************************
    Returns a cleaned version of 'current_path'
    **************************************************************************/
    fn get_clean_path current_path =
    (
        local clean_path = substituteString current_path "\"" ""
        
        if clean_path != "" do
            clean_path = pathconfig.normalizePath clean_path
        
        clean_path --return value
    ), --end fn
    
    /**************************************************************************
    Returns all sub directories of the root directory 'root'
    **************************************************************************/
	fn get_sub_directories root =
	(
		local sub_directories = getDirectories (root + "\\*")
		
		for current_directory in sub_directories do
			join sub_directories (getDirectories (current_directory + "\\*"))
			
		sub_directories --return value
	), --end fn
    
    /**************************************************************************
    Returns all maxscript files in all directories 'search_directories'
    **************************************************************************/
	fn get_maxscript_files search_directories =
	(
        local maxscript_files = #()
        
		for current_directory in search_directories do
        (
            found_files = getFiles (current_directory + "\\*.ms")
            
            if found_files.count > 0 do
                join maxscript_files found_files
        ) --end for
        
        maxscript_files --return value
	), --end fn
    
    /**************************************************************************
    Stores the display name for all found maxscript files
    **************************************************************************/
    fn collect_display_names root_directory =
    (
        this.display_names = for current in this.maxscript_files collect \
                                substituteString current (root_directory + "\\") ""
    ), --end fn
    
    /**************************************************************************
    Stores all maxscript files in the root directory
    **************************************************************************/
    fn collect_maxscript_files root_directory include_sub_directories =
    (
        local search_directories = #(root_directory)
        
        if include_sub_directories do
            join search_directories (this.get_sub_directories root_directory)
        
        this.maxscript_files = this.get_maxscript_files search_directories
        sort this.maxscript_files
        
        this.collect_display_names root_directory
    ), --end fn
    
    /**************************************************************************
    Encrypts maxscript files based on index. Returns true on success
    **************************************************************************/
    fn encrypt_files script_indices encrypt_version =
    (
        local success = true
        
        for index in script_indices do
        (
            try
            (
                encryptScript this.maxscript_files[index] version:encrypt_version
            ) --end try
            catch
            (
                print("ERROR: Could not encrypt file " + this.maxscript_files[index])
                success = false
            ) --end catch
        ) --end for
        
        success --return value
    ), --end fn
    
    /**************************************************************************
    Moves encrypted files. Returns true on success
    **************************************************************************/
    fn move_encrypted_files source_directory destination_directory script_indices =
    (
        local success = true
        
        for index in script_indices do
        (
            source_path = this.maxscript_files[index] + "e"
            destination_path = substituteString source_path source_directory destination_directory
            
            if not (makeDir (getFilenamePath destination_path) all:true) do
            (
                print("ERROR: Could not create directory " + (getFilenamePath destination_path))
                success = false
            ) --end if
            
            if (doesFileExist destination_path) do
            (
                if not (deleteFile destination_path) do
                (
                    print("ERROR: Could not delete old file " + destination_path)
                    success = false
                ) --end if
            ) --end if
            
            if not (renameFile source_path destination_path) do
            (
                print("ERROR: Could not move file " + source_path + " to " + destination_path)
                success = false
            ) --end if
        ) --end for
        
        success --return value
    ) --end fn
) --end struct


( --local scope
    global encrypt_script_rollout
    local encrypt_script_ini = getDir #plugcfg + "\\EncryptScripts.ini"
	local encrypt_script_position

	rollout encrypt_script_rollout "Encrypt Scripts"
	(
        local encrypter = EncryptScripts()

        /**********************************************************************
        Updates the script list box. Returns the total number scripts found
        **********************************************************************/
        fn refresh_script_list =
        (
            local root_directory = encrypt_script_rollout.script_directory_text.text
            local include_sub = encrypt_script_rollout.sub_directory_checkbox.checked
            local total_scripts = 0
            
            if (doesFileExist root_directory) then
            (
                encrypter.collect_maxscript_files root_directory include_sub
                encrypt_script_rollout.script_listbox.items = encrypter.display_names
                total_scripts = encrypter.maxscript_files.count
            ) --end if
            else
                encrypt_script_rollout.script_listbox.items = #()
            
            total_scripts --return value
        ) --end fn
        
        groupBox script_group "Script Directory" pos:[5,5] width:440 height:65
            editText script_directory_text "" pos:[13,25] width:372 height:17
            button script_browse_button "..." pos:[395,24] width:40 height:20
            checkbox sub_directory_checkbox "Search Sub Directories" pos:[15,50] width:150 height:15
        
        groupBox file_group "Scripts" pos:[5,75] width:440 height:200
            multiListBox script_listbox "" pos:[15,95] width:420 height:11
            button select_all_button "Select All" pos:[15,250] width:100 height:20
            button select_invert_button "Invert Selection" pos:[125,250] width:100 height:20
            button select_none_button "Select None" pos:[235,250] width:100 height:20
        
        groupBox output_group "Output Directory (Optional)" pos:[5,280] width:440 height:45
            editText output_directory_text "" pos:[15,300] width:370 height:17
            button output_browse_button "..." pos:[395,299] width:40 height:20
        
        groupBox option_group "Options" pos:[5,330] width:440 height:45
            label encrypt_label "Encryption:" pos:[15,350] width:70 height:15
            radioButtons encrypt_mode_radio "" pos:[90,350] labels:#("Version 0   ", "Version 1") default:2 columns:2
        
        button encrypt_button "Encrypt Selected Scripts" pos:[5,385] width:440 height:20
        label status_label "INFO: Standing by..." pos:[5,415] width:440 height:15
        
        /**********************************************************************
        UI open event handler. Prepares UI for first time open
        **********************************************************************/
        on encrypt_script_rollout open do
        (
            local ini_value = getIniSetting encrypt_script_ini "Dialog" "Script_Directory"
            if (doesFileExist ini_value) do
                script_directory_text.text = ini_value
            
            ini_value = getIniSetting encrypt_script_ini "Dialog" "Include_Sub_Directories"
            if (toLower ini_value) == "true" do
                sub_directory_checkbox.checked = true
            
            ini_value = getIniSetting encrypt_script_ini "Dialog" "Output_Directory"
            if (doesFileExist ini_value) do
                output_directory_text.text = ini_value
            
            ini_value = getIniSetting encrypt_script_ini "Dialog" "Encryption_Version"
            if ini_value == "1" do
                encrypt_mode_radio.state = 1
            
            total_scripts = refresh_script_list()
            status_label.text = "INFO: " + (total_scripts as string) + " files found"
        ) --end on
        
        /**********************************************************************
        UI move event handler. Stores the current position of the UI
        **********************************************************************/
        on encrypt_script_rollout moved pos do
            setIniSetting encrypt_script_ini "Dialog" "Position" (pos as string)
        
        /**********************************************************************
        UI close event handler. Stores paths of the UI for next launch
        **********************************************************************/
        on encrypt_script_rollout close do
        (
            setIniSetting encrypt_script_ini "Dialog" "Script_Directory" script_directory_text.text
            setIniSetting encrypt_script_ini "Dialog" "Include_Sub_Directories" (sub_directory_checkbox.checked as string)
            setIniSetting encrypt_script_ini "Dialog" "Output_Directory" output_directory_text.text
            setIniSetting encrypt_script_ini "Dialog" "Encryption_Version" (encrypt_mode_radio.state as string)
        ) --end on
        
        /**********************************************************************
        Script directory changed event handler. Cleans the script directory
        text and updates the script list
        **********************************************************************/
        on script_directory_text changed the_path do
        (
            local clean_path = encrypter.get_clean_path the_path
            script_directory_text.text = clean_path
            
            local total_scripts = refresh_script_list()
            
            status_label.text = "INFO: " + (total_scripts as string) + " files found"
        ) --end on
        
        /**********************************************************************
        Script browse pressed event handler. Opens a dialog allowing the user
        to select a script directory
        **********************************************************************/
        on script_browse_button pressed do
        (
            local script_path = getSavePath caption:"Script Directory"
            
			if script_path != undefined then
			(
				script_directory_text.text = script_path
				local total_scripts = refresh_script_list()
            
                status_label.text = "INFO: " + (total_scripts as string) + " files found"
			)--end if
            else
                status_label.text = "WARNING: Script directory selection cancelled"
        ) --end on
        
        /**********************************************************************
        Include sub directories changed event handler. Refreshes the script
        list to include/exclude sub directories
        **********************************************************************/
        on sub_directory_checkbox changed the_state do
        (
            local total_scripts = refresh_script_list()
            
            status_label.text = "INFO: " + (total_scripts as string) + " files found"
        ) --end on
        
        /**********************************************************************
        Select all button clicked event handler. Selects all scripts in the
        script listbox
        **********************************************************************/
		on select_all_button pressed do
        (
            local index_selection = #{1..(script_listbox.items.count)}
            script_listbox.selection = index_selection

            status_label.text = "INFO: All scripts have been selected"
        ) --end on

        /**********************************************************************
        Select invert button clicked event handler. Selects all scripts in the
        script listbox that are not selected and deselects selected scripts
        **********************************************************************/
		on select_invert_button pressed do
        (
            local index_selection = #{1..(script_listbox.items.count)}
            index_selection = index_selection - script_listbox.selection
            script_listbox.selection = index_selection

            status_label.text = "INFO: Script selection has been inverted"
        ) --end on

        /**********************************************************************
        Select none button clicked event handler. Deselects all scripts in the
        script listbox
        **********************************************************************/
		on select_none_button pressed do
        (
            script_listbox.selection = #{}

            status_label.text = "INFO: All scripts have been deselected"
        ) --end on
        
        /**********************************************************************
        Output directory changed event handler. Cleans the Output directory
        text
        **********************************************************************/
        on output_directory_text changed the_path do
        (
            local clean_path = encrypter.get_clean_path the_path
            output_directory_text.text = clean_path
            
            status_label.text = "INFO: Output directory has been set"
        ) --end on
        
        /**********************************************************************
        Output browse pressed event handler. Opens a dialog allowing the user
        to select an output directory
        **********************************************************************/
        on output_browse_button pressed do
        (
            local output_path = getSavePath caption:"Output Directory"
            
			if output_path != undefined then
			(
				output_directory_text.text = output_path
                
                status_label.text = "INFO: Output directory has been set"
			)--end if
            else
                status_label.text = "WARNING: Output directory selection cancelled"
        ) --end on
        
        /**********************************************************************
        Encrypt script pressed event handler. Encrypts the selected scripts
        and optionally moves them to the output directory
        **********************************************************************/
        on encrypt_button pressed do
        (
            local script_selection = script_listbox.selection as array
            local encrypt_version = encrypt_mode_radio.state - 1
            local encrypt_success = true
            local move_success = true
            local message = ""
            
            if script_selection.count == 0 do
            (
                status_label.text = "ERROR: No scripts selected"
                return false
            ) --end if
            
            if output_directory_text.text != "" do
            (
                if not (doesFileExist output_directory_text.text) do
                (
                    status_label.text = "ERROR: Output directory does not exist"
                    return false
                ) --end if
            ) --end if
            
            status_label.text = "INFO: Encrypting scripts. Please wait..."
            if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
            
            encrypt_success = encrypter.encrypt_files script_selection encrypt_version
            
            if output_directory_text.text != "" do
                move_success = encrypter.move_encrypted_files script_directory_text.text output_directory_text.text script_selection
            
            if encrypt_success and move_success then
                message = "INFO: Encryption complete. 0 errors"
            else
                message = "WARNING: Encryption complete. Errors occurred. Check listener"
            
            status_label.text = message
        ) --end on
    ) --end rollout
    
    encrypt_script_position = execute (getIniSetting encrypt_script_ini "Dialog" "Position")
	if encrypt_script_position == OK do encrypt_script_position = [100,100]
	if encrypt_script_position.x > sysinfo.desktopsize.x - 450 do
        encrypt_script_position.x = sysinfo.desktopsize.x - 455
	if encrypt_script_position.y > sysinfo.desktopsize.y - 490 do
        encrypt_script_position.y = sysinfo.desktopsize.y - 490
	if encrypt_script_position.x < 0 do encrypt_script_position.x = 0 
	if encrypt_script_position.y < 0 do encrypt_script_position.y = 0
	createDialog encrypt_script_rollout 450 430 encrypt_script_position.x encrypt_script_position.y
) --end scope
